##########################################################
# A toy Monte Carlo simulation to show:                  #
#  1) How does nonresponse bias arise                    #
#  2) How it is possible to reduce it by calibration     #
#     2.1) at Info-U level                               #
#     2.2) at Info-S level                               #
#                                                        #
# Info-U: Calibration totals are ACTUAL KNOWN population #
#         totals.                                        #
# Info-S: Calibration totals are UNBIASED ESTIMATES of   #
#         population totals of variables that are known  #
#         BOTH for respondents AND for non-respondents.  #
##########################################################


#######################################################################
# For illustration purposes, we will use an artificial population 'U' #
# of N = 2000 units, with the following 3 variables:                  #
# - id:     an integer identifier                                     #
# - sex:    a factor with levels 'f', 'm'                             #
# - income: a numeric                                                 #
#                                                                     #
# Furthermore, we will assume what follows:                           #
# 1) We want to estimate the total income I of U                      #
# 2) U contains nearly the same number of males and females:          #
#    Nm ~ Nf                                                          #
# 3) Response probability for males is significantly smaller than for #
#    females, say half:                                               #
#    phim = 0.4 = phif  / 2                                           #
# 4) Incomes are higher, on average, for males than for females, say  #
#    twice:                                                           #
#    Im / Nm ~ 2 If / Nf                                              #
# 5) We use simple random sampling without replacement (srs) to draw  #
#    samples from U                                                   #
#                                                                     #
# Question:                                                           #
# - Under assumptions above, how much bias should we expect if we     #
#   decide to use a standard HT estimator?                            #
# Answer:                                                             #
# - It's easy to prove theoretically that estimated total income will #
#   be about (2/3)*phif*I = (8/15)*I, namely only ~ 0.53 of the true  #
#   total value I.                                                    #
#   The same result will also emerge numerically from the simulation. #
#######################################################################


#####################################################
# Load our already prepared artificial population U #
#####################################################
load("NRBias.RData")

# Have a look at U
head(U)
summary(U)

# Compute relevant population statistics and verify that U actually complies
# with assumptions 2) and 4) above
N <- nrow(U)
Counts.sex <- table(U$sex)
Counts.sex
Total.income <- sum(U$income)
Total.income
Total.income.sex <- tapply(U$income, U$sex, mean) 
Total.income.sex

##############################################################
# Define the sampling design and the sample size to be later #
# used in the Monte Carlo simulation, in accordance with     #
# assumption 5) above                                        #
##############################################################
# We will select simple random samples without replacement (srs) from U
# Sample size n = 100
n <- 100
# Sampling fraction (=inclusion probability for srs)
f <- n/N
f
# Direct weights (=inverse inclusion probabilities) d = 1/p = 1/f
d <- N/n
d
# Add convenience columns to U
U <- cbind(U, d = d, f = f)
# Have a look at U again
head(U)

###############################################################
# Now, according to assumption 3) above, set smaller response #
# probabilities for males than for females:                   #
# - phim = 0.4                                                #
# - phif = 0.8                                                #
###############################################################
phi <- c(phif = 0.8, phim = 0.4)
phi
# Tie response probabilities to population units, according to sex:
U$phi <- phi[unclass(U$sex)]
# Have a look at U again
head(U)



#####################################################################
# Easy things first: Before undertaking the Monte Carlo simulation  #
#                    let us perform a 'bias analysis' based on just #
#                    a single sample of respondents.                #
#####################################################################
## First, select a single sample 's' fron U under srs
s <- U[sample(1:N, n, rep = FALSE), ]

## Then, apply a random response mechanism to s and obtain the (smaller)
## respondent sample 'r'
r <- s[runif(nrow(s)) < s$phi, ]
# Actual size of response set is:
m <- nrow(r)
m

## Now let's start computing estimates based on the response set r, and carry
## out our bias analysis
# Bind survey data and sampling design metadata
require(ReGenesees)
rdes <- e.svydesign(r, ids = ~id, weights = ~d, fpc = ~f)
rdes

# Estimated sex distribution is obviously biased due to nonresponse (HT stands
# for Horvitz-Thompson):
sex.HT <- svystatTM(rdes, ~sex)
sex.HT
# Indeed, true sex counts were:
Counts.sex

# Since income is strongly correleted with sex, nonresponse bias 'transfers'
# from sex to income:
income.HT <- svystatTM(rdes, ~income)
income.HT
# Indeed, true income total was:
Total.income

# NOTE: This is our FIRST LESSON LEARNED
#       Bias tends to "propagate" across estimates, owing to the correlation
#       structure of the variables of interest

# NOTE: As anticipated, the HT estimate of the total of income should be only
#       about (2/3)*phif = (8/15) ~ 0.53 of the correspondind true population
#       total
coef(income.HT)/Total.income


## Now let's switch to calibration.
## Nonresponse bias can be reduced by calibrating on auxiliary variables which
## are able to explain (at least partially) the response propensity.

# In the present example, the population distribution of sex is perfectly fit to
# the scope, thus let's calibrate accordingly.

## FIRST SOLUTION: Info-U calibration ##
## Sex counts are actual KNOWN totals, derived from U.

# Prepare the known total template:
popU <- pop.template(data = rdes, calmodel = ~sex - 1)
popU

# Fill the template by using U as universe:
popU <- fill.template(universe = U, template = popU)
popU

# Lastly calibrate:
rcalU <- e.calibrate(design = rdes, df.population = popU)
rcalU

# By construction, calibration exactly removes nonresponse bias from the
# estimated sex distribution (CALU stands for Info-U Calibration):
sex.CALU <- svystatTM(rcalU, ~sex)
sex.CALU
# Indeed, true sex counts were:
Counts.sex

# The good news is that calibration did a pretty good job for income too,
# significantly reducing nonresponse bias:
income.CALU <- svystatTM(rcalU, ~income)
income.CALU
# Indeed, true income total was:
Total.income
# So that we are left with a quite small relative bias:
coef(income.CALU)/Total.income

## SECOND SOLUTION: Info-S calibration ##
## Sex counts are NO LONGER KNOWN totals, but rather unbiased HT estimates
## derived from the planned sample s.
## NOTE: This is only possible if we are ablo to register sex information for
##       BOTH respondents AND non-respondents, i.e. before refusal.

# Prepare the known total template:
popS <- pop.template(data = rdes, calmodel = ~sex - 1)
popS

# Fill the template with HT estimates derived from s:
  # 1. Bind planned sample data s and sampling design metadata
sdes <- e.svydesign(s, ids = ~id, weights = ~d, fpc = ~f)
  # 2. Compute HT estimates of sex counts from sdes and fill popS
popS <- aux.estimates(design = sdes, template = popS)
popS

# Lastly calibrate:
rcalS <- e.calibrate(design = rdes, df.population = popS)
rcalS

# Since 'known' totals are UNBIASED HT estimates, calibration reduces the
# nonresponse bias affecting the estimated sex distribution (CALS stands for
# Info-S Calibration):
sex.CALS <- svystatTM(rcalS, ~sex)
sex.CALS
# Indeed, true sex counts were:
Counts.sex

# The good news is that AGAIN calibration did a pretty good job for income too,
# significantly reducing nonresponse bias (though NOT AS MUCH as with Info-U):
income.CALS <- svystatTM(rcalS, ~income)
income.CALS
# Indeed, true income total was:
Total.income
# So that we are left with a much smaller relative bias than before:
coef(income.CALS)/Total.income


# NOTE: This is our SECOND LESSON LEARNED
#       Calibration removes any bias possibly affecting the auxiliary variables
#       (perfectly or nearly, depending on whether Info-U or Info-S is used).
#       As a remarkable side effect, calibration at least decreases the bias
#       affecting interest variables that are correlated to the auxiliary ones

# NOTE: It should be clear that NOT ALL THE AUXILIARY VARIABLES ARE EQUALLY
#       GOOD.
#       As noted by Sarndal and Lundstrom, POWERFUL auxiliary variables should:
#       1) Be able to explain the variation of response propensity
#       2) Covary with the main study variables
#       3) Identify the most important estimation domains



###################################################################
# Run our toy Monte Carlo experiment with a bunch of 200 samples. #
###################################################################
nsamp <- 200

# Prepare vectors to store HT and CAL estimates
income.HT  <- vector(mode = "numeric", length = nsamp)
income.CALU <- vector(mode = "numeric", length = nsamp)
income.CALS <- vector(mode = "numeric", length = nsamp)

# Run Monte Carlo loop
for (i in 1:nsamp){
     ## Select full-response sample s fron U under srs
     s <- U[sample(1:N, n, rep = FALSE), ]

     ## Bind full-sample data s and sampling design metadata
     sdes <- e.svydesign(s, ids = ~id, weights = ~d, fpc = ~f)

     ## Apply random response mechanism to s and obtain r
     r <- s[runif(nrow(s)) < s$phi, ]

     ## Bind respondents sample r and sampling design metadata
     rdes <- e.svydesign(r, ids = ~id, weights = ~d, fpc = ~f)

     ## Compute HT estimate of total income
     income.HT[i] <- coef(svystatTM(rdes, ~income))

     ## Calibrate at Info-U level (reuse the population totals popU built above)
     rcalU <- e.calibrate(design = rdes, df.population = popU)

     ## Compute CAL estimate of total income
     income.CALU[i] <- coef(svystatTM(rcalU, ~income))

     ## Calibrate at Info-S level (cannot reuse the population totals popS
     ## built above, since HT estimates based on s CHANGE from sample to sample)
     popS <- aux.estimates(design = sdes, calmodel = ~sex - 1)
     rcalS <- e.calibrate(design = rdes, df.population = popS)

     ## Compute CAL estimate of total income
     income.CALS[i] <- coef(svystatTM(rcalS, ~income))
    }


	
###############################################################
# Compare MC generated HT and CAL estimates via scatter plots #
###############################################################
# RED POINTS -> HT estimates: bias is evident and variance is large
plot(income.HT, ylim = c(0.2*Total.income, 1.2*Total.income), pch = 19, col = "red", ylab = "Estimates", main = "Estimates of Total Income\nHT (red) and CAL (blue = Info-U, green = Info-S)")
# RED HORIZ. LINE -> True value of total income, a reference to assess the
#                    size of nonresponse bias
abline(h = Total.income, col = "red", lwd = 2)
# GREEN POINTS -> CAL estimates at Info-S level: bias is removed but variance
#                 is only slightly reduced, compared to HT
points(income.CALS, col = "green", pch = 19)

# BLUE POINTS -> CAL estimates at Info-U level: bias is removed and variance is
#                significantly reduced, compared to HT
points(income.CALU, col = "blue", pch = 19)


# An additional plot to clarify (if needed) that the variance of Info-U CAL
# estimates is not actually as small as it seemed in the previous plot for
# reasons of scale
# BLUE POINTS -> CAL estimates at Info-U level
plot(income.CALU, pch = 19, col = "blue", ylab = "Estimates", main = "Calibration Estimates of Total Income (Info-U)")
# RED HORIZ. LINE -> True value of total income: a reference to assess estimates
# variability
abline(h = Total.income, col = "red", lwd = 2)

